#! /usr/bin/env python3
###############################################################################
#
# Copyright 2022
# ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
###############################################################################
# Author: Reinier vd Walle
# Purpose: Generate the hex files for RAM initialization in SDP designs
###############################################################################

# The sdp_hex.py files are generated in sdp/python and have to be manually
# copied to the design directory in git:
# . applications/lofar2/designs/lofar2_unb2b_sdp_station/tb/data
# . applications/lofar2/designs/lofar2_unb2c_sdp_station/tb/data
# From there the revision hdllib.cfg will copy the files the project build
# data directory for quartus and for modelsim.
#
# The IntelHex module outputs the addresses as byte addresses that increment
# by 4 for every line. Instead the memory in synthesis requires the addresses
# as word addresses that increment by 1. For simulation generated file can be
# used tb/data. For synthesis the word addresses file needs to be used from
# src/data and can be obtained using the GUI:
#
# 1 Generate hex files with:
#   > python sdp_hex.py
#   Copy file to the tb/data/
#   ==> equal diff (idem for unb2b)
#   diff bf_unit_ss_wide_0.hex ../../../../designs/lofar2_unb2c_sdp_station/tb/data/bf_unit_ss_wide_0.hex
#   diff bf_unit_weights_0.hex ../../../../designs/lofar2_unb2c_sdp_station/tb/data/bf_unit_weights_0.hex
#   diff gains_1024_complex_16b13f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/tb/data/gains_1024_complex_16b13f_unit_0.hex
#   diff gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/tb/data/gains_1024_complex_16b14f_unit_0.hex
#
# Remark: All index files are identical, because they all have the same
#         defaults. So only need to convert index file 0.
#         . Keep generated byte address file with index 0 in byte/
#         . Keep converted word address file with index 0 in word/
#
# 2 Start Quartus GUI to convert byte address file into word addres hex file:
#   Menu File --> Open one hex file
#   . A prompt will pop up to enter the word size, enter 8 (default) -> OK.
#   Menu Edit -> Memory Size Wizard
#   . Enter word size 32 -> Next
#   . Select Combine existing words
#   . Keep number of words default (for gains_1024_complex ... .hex, it will be 1024) -> Finish
#   . File -> Save As in temporary location word/ (do not create new project file)
#   . File -> Close
#   Copy hex files to src/data/ in:
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/tb/data/gains_1024_complex_16b14f_unit_0.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/tb/data/gains_1024_complex_16b14f_unit_1.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/tb/data/gains_1024_complex_16b14f_unit_2.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/tb/data/gains_1024_complex_16b14f_unit_3.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/tb/data/gains_1024_complex_16b14f_unit_4.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/tb/data/gains_1024_complex_16b14f_unit_5.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/tb/data/gains_1024_complex_16b14f_unit_6.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/tb/data/gains_1024_complex_16b14f_unit_7.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/tb/data/gains_1024_complex_16b14f_unit_8.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/tb/data/gains_1024_complex_16b14f_unit_9.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/tb/data/gains_1024_complex_16b14f_unit_10.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/tb/data/gains_1024_complex_16b14f_unit_11.hex
#
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/tb/data/gains_1024_complex_16b14f_unit_0.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/tb/data/gains_1024_complex_16b14f_unit_1.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/tb/data/gains_1024_complex_16b14f_unit_2.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/tb/data/gains_1024_complex_16b14f_unit_3.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/tb/data/gains_1024_complex_16b14f_unit_4.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/tb/data/gains_1024_complex_16b14f_unit_5.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/tb/data/gains_1024_complex_16b14f_unit_6.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/tb/data/gains_1024_complex_16b14f_unit_7.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/tb/data/gains_1024_complex_16b14f_unit_8.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/tb/data/gains_1024_complex_16b14f_unit_9.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/tb/data/gains_1024_complex_16b14f_unit_10.hex
#     cp byte/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/tb/data/gains_1024_complex_16b14f_unit_11.hex
#
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/src/data/gains_1024_complex_16b14f_unit_0.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/src/data/gains_1024_complex_16b14f_unit_1.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/src/data/gains_1024_complex_16b14f_unit_2.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/src/data/gains_1024_complex_16b14f_unit_3.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/src/data/gains_1024_complex_16b14f_unit_4.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/src/data/gains_1024_complex_16b14f_unit_5.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/src/data/gains_1024_complex_16b14f_unit_6.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/src/data/gains_1024_complex_16b14f_unit_7.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/src/data/gains_1024_complex_16b14f_unit_8.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/src/data/gains_1024_complex_16b14f_unit_9.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/src/data/gains_1024_complex_16b14f_unit_10.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2b_sdp_station/src/data/gains_1024_complex_16b14f_unit_11.hex
#
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/src/data/gains_1024_complex_16b14f_unit_0.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/src/data/gains_1024_complex_16b14f_unit_1.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/src/data/gains_1024_complex_16b14f_unit_2.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/src/data/gains_1024_complex_16b14f_unit_3.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/src/data/gains_1024_complex_16b14f_unit_4.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/src/data/gains_1024_complex_16b14f_unit_5.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/src/data/gains_1024_complex_16b14f_unit_6.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/src/data/gains_1024_complex_16b14f_unit_7.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/src/data/gains_1024_complex_16b14f_unit_8.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/src/data/gains_1024_complex_16b14f_unit_9.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/src/data/gains_1024_complex_16b14f_unit_10.hex
#     cp word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/src/data/gains_1024_complex_16b14f_unit_11.hex
#
#   diff word/bf_unit_ss_wide_0.hex ../../../../designs/lofar2_unb2c_sdp_station/src/data/bf_unit_ss_wide_0.hex
#   diff word/bf_unit_weights_0.hex ../../../../designs/lofar2_unb2c_sdp_station/src/data/bf_unit_weights_0.hex
#   diff word/gains_1024_complex_16b13f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/src/data/gains_1024_complex_16b13f_unit_0.hex
#   diff word/gains_1024_complex_16b14f_unit_0.hex ../../../../designs/lofar2_unb2c_sdp_station/src/data/gains_1024_complex_16b14f_unit_0.hex
#
# TODO: Try use the converted files from word/ also for simulation in tb/data,
#       because there should be no difference between sim and synth.

from intelhex import IntelHex
import struct

# Subband select
for i in range(0,6):
    f = IntelHex()    
    for j in range(0,2*488):
        #f.puts(j*4, struct.pack(">I",j))
        f.puts(j*2, struct.pack(">H",j))

    f.write_hex_file("bf_unit_ss_wide_"+str(i)+".hex", byte_count = 2)

# Weigths are 16b complex value so 4 byte words, Im in byte 0, 1 and Re in
# byte 2, 3, e.g.:
#                 0  1  2  3
#   2**14 + 0j : 00 00 40 00    , 2**14 = 0x4000
#   2**13 + 0j : 00 00 20 00    , 2**13 = 0x2000

# BF weights
for i in range(0,12):
    f = IntelHex()    
    for j in range(0,2*488):
        f[4*j] = 0x00
        f[4*j + 1] = 0x00
        f[4*j + 2] = 0x40
        f[4*j + 3] = 0x00
    f.write_hex_file("bf_unit_weights_"+str(i)+".hex", byte_count = 4)

# Subband weights
for i in range(0,6):
    f = IntelHex()
    for j in range(0,1024):
        f[4*j] = 0x00
        f[4*j + 1] = 0x00
        f[4*j + 2] = 0x20
        f[4*j + 3] = 0x00
    f.write_hex_file("gains_1024_complex_16b13f_unit_"+str(i)+".hex", byte_count = 4)

for i in range(0,6):
    f = IntelHex()
    for j in range(0,1024):
        f[4*j] = 0x00
        f[4*j + 1] = 0x00
        f[4*j + 2] = 0x40
        f[4*j + 3] = 0x00
    f.write_hex_file("gains_1024_complex_16b14f_unit_"+str(i)+".hex", byte_count = 4)

